@@ -14,14 +14,16 @@ import sys
 import typing
 from collections.abc import Iterator
 from contextlib import contextmanager
+from contextlib import nullcontext
 from functools import partial
 from subprocess import CompletedProcess
 from types import FunctionType
 from types import GenericAlias
 from typing import Any
 from typing import cast
+from typing import ContextManager
+from typing import NoReturn
 from typing import TYPE_CHECKING
-from typing import TypedDict
 
 import requests
 import rich
@@ -33,6 +35,11 @@ from ptscripts import process
 from ptscripts.virtualenv import VirtualEnv
 from ptscripts.virtualenv import VirtualEnvConfig
 
+if sys.version_info < (3, 11):
+    from typing_extensions import TypedDict, NotRequired
+else:
+    from typing import TypedDict, NotRequired
+
 try:
     import importlib.metadata
 
@@ -54,14 +61,15 @@ class ArgumentOptions(TypedDict):
     TypedDict class documenting the acceptable keys and their types for arguments.
     """
 
-    flags: list[str]
-    help: str
-    action: str | argparse.Action
-    nargs: int | str
-    const: Any
-    choices: list[str]
-    required: bool
-    metavar: str
+    help: NotRequired[str]
+    flags: NotRequired[list[str]]
+    action: NotRequired[str | argparse.Action]
+    nargs: NotRequired[int | str]
+    const: NotRequired[Any]
+    choices: NotRequired[list[str] | tuple[str, ...]]
+    required: NotRequired[bool]
+    metavar: NotRequired[str]
+    default: NotRequired[Any]
 
 
 class FullArgumentOptions(ArgumentOptions):
@@ -71,7 +79,6 @@ class FullArgumentOptions(ArgumentOptions):
 
     dest: str
     type: type[Any]
-    default: Any
 
 
 class Context:
@@ -135,7 +142,7 @@ class Context:
         """
         self.console.log(*args, style="log-error", _stack_offset=2)
 
-    def exit(self, status=0, message=None):
+    def exit(self, status=0, message=None) -> NoReturn:  # type: ignore[misc]
         """
         Exit the command execution.
         """
@@ -156,7 +163,7 @@ class Context:
         capture: bool = False,
         interactive: bool = False,
         **kwargs,
-    ) -> CompletedProcess[str]:
+    ) -> CompletedProcess[bytes]:
         """
         Run a subprocess.
         """
@@ -179,7 +186,7 @@ class Context:
         capture: bool = False,
         interactive: bool = False,
         **kwargs,
-    ) -> CompletedProcess[str] | None:
+    ) -> CompletedProcess[bytes]:
         """
         Run a subprocess.
 
@@ -206,9 +213,11 @@ class Context:
                 **kwargs,
             )
         except subprocess.CalledProcessError as exc:
-            self.error(str(exc))
-            self.exit(exc.returncode)
-        return None
+            self._exit(str(exc), exc.returncode)
+
+    def _exit(self, msg: str, returncode: int) -> NoReturn:
+        self.error(msg)
+        self.exit(returncode)
 
     @contextmanager
     def chdir(self, path: pathlib.Path) -> Iterator[pathlib.Path]:
@@ -248,13 +257,13 @@ class Context:
         return requests.Session()
 
 
-class RegisteredImports:
+class DefaultVirtualenvConfig:
     """
     Simple class to hold registered imports.
     """
 
-    _instance: RegisteredImports | None = None
-    _registered_imports: list[str]
+    _instance: DefaultVirtualenvConfig | None = None
+    venv_config: VirtualEnvConfig
 
     def __new__(cls):
         """
@@ -262,24 +271,56 @@ class RegisteredImports:
         """
         if cls._instance is None:
             instance = super().__new__(cls)
-            instance._registered_imports = []
             cls._instance = instance
         return cls._instance
 
     @classmethod
-    def register_import(cls, import_module: str) -> None:
+    def set_default_venv_config(cls, venv_config: VirtualEnvConfig) -> None:
+        """
+        Register an import.
+        """
+        instance = cls._instance
+        if instance is None:
+            instance = cls()
+        if venv_config and "name" not in venv_config:
+            venv_config["name"] = "default"
+        instance.venv_config = venv_config
+
+
+class RegisteredImports:
+    """
+    Simple class to hold registered imports.
+    """
+
+    _instance: RegisteredImports | None = None
+    _registered_imports: dict[str, VirtualEnvConfig | None]
+
+    def __new__(cls):
+        """
+        Method that instantiates a singleton class and returns it.
+        """
+        if cls._instance is None:
+            instance = super().__new__(cls)
+            instance._registered_imports = {}
+            cls._instance = instance
+        return cls._instance
+
+    @classmethod
+    def register_import(
+        cls, import_module: str, venv_config: VirtualEnvConfig | None = None
+    ) -> None:
         """
         Register an import.
         """
         instance = cls()
         if import_module not in instance._registered_imports:
-            instance._registered_imports.append(import_module)
+            instance._registered_imports[import_module] = venv_config
 
     def __iter__(self):
         """
         Return an iterator of all registered imports.
         """
-        return iter(self._registered_imports)
+        return iter(self._registered_imports.items())
 
 
 class Parser:
@@ -370,14 +411,29 @@ class Parser:
         return cls._instance
 
     def _process_registered_tool_modules(self):
-        for module_name in RegisteredImports():
-            try:
-                importlib.import_module(module_name)
-            except ImportError as exc:
-                if os.environ.get("TOOLS_IGNORE_IMPORT_ERRORS", "0") == "0":
-                    self.context.warn(
-                        f"Could not import the registered tools module {module_name!r}: {exc}"
-                    )
+        default_venv: VirtualEnv | ContextManager[None]
+        default_venv_config = DefaultVirtualenvConfig().venv_config
+        if default_venv_config:
+            default_venv = VirtualEnv(ctx=self.context, **default_venv_config)
+        else:
+            default_venv = nullcontext()
+        with default_venv:
+            for module_name, venv_config in RegisteredImports():
+                venv: VirtualEnv | ContextManager[None]
+                if venv_config:
+                    if "name" not in venv_config:
+                        venv_config["name"] = module_name
+                    venv = VirtualEnv(ctx=self.context, **venv_config)
+                else:
+                    venv = nullcontext()
+                with venv:
+                    try:
+                        importlib.import_module(module_name)
+                    except ImportError as exc:
+                        if os.environ.get("TOOLS_IGNORE_IMPORT_ERRORS", "0") == "0":
+                            self.context.warn(
+                                f"Could not import the registered tools module {module_name!r}: {exc}"
+                            )
 
     def parse_args(self):
         """
@@ -415,14 +471,14 @@ class Parser:
 
 class GroupReference:
     """
-    Simple class to hold tools command group names comparable to how
-    they would be invoked using the CLI.
+    Simple class to hold tools command group names.
 
-    For example, tools vm create is stored as ("tools", "vm", "create")
+    These are comparable to how they would be invoked using the CLI.
+    For example, ``tools vm create`` is stored as ``("tools", "vm", "create")``
     """
 
     _instance: GroupReference | None = None
-    _commands: list[str]
+    _commands: dict[tuple[str, ...], CommandGroup]
 
     def __new__(cls):
         """
@@ -435,7 +491,7 @@ class GroupReference:
         return cls._instance
 
     @classmethod
-    def add_command(cls, cli_name: tuple[str], group: CommandGroup) -> None:
+    def add_command(cls, cli_name: tuple[str, ...], group: CommandGroup) -> None:
         """
         Add a tools command.
         """
@@ -471,6 +527,8 @@ class CommandGroup:
             GroupReference.add_command(tuple(parent + [name]), self)
             parent = GroupReference()[tuple(parent)]
 
+        if venv_config and "name" not in venv_config:
+            venv_config["name"] = self.name
         self.venv_config = venv_config or {}
         self.parser = parent.subparsers.add_parser(
             name.replace("_", "-"),
@@ -602,7 +660,7 @@ class CommandGroup:
                     if not kwargs["help"].endswith("."):
                         kwargs["help"] += "."
                     kwargs["help"] += " [default: %(default)s]"
-            flags = kwargs.pop("flags", None)  # type: ignore[misc]
+            flags = kwargs.pop("flags", None)
             if flags is None:
                 flags = [f"--{parameter.name.replace('_', '-')}"]
             log.debug("Adding Command %r. Flags: %s; KwArgs: %s", name, flags, kwargs)
@@ -634,22 +692,22 @@ class CommandGroup:
                     kwargs[name] = getattr(options, name)
 
         bound = signature.bind_partial(*args, **kwargs)
-        venv = None
+        venv: VirtualEnv | ContextManager[None]
         if venv_config:
-            venv_name = getattr(options, f"{self.name}_command")
-            venv = VirtualEnv(name=f"{self.name}.{venv_name}", ctx=self.context, **venv_config)
+            if "name" not in venv_config:
+                venv_config["name"] = getattr(options, f"{self.name}_command")
+            venv = VirtualEnv(ctx=self.context, **venv_config)
         elif self.venv_config:
-            venv = VirtualEnv(name=self.name, ctx=self.context, **self.venv_config)
-        if venv:
-            with venv:
-                previous_venv = self.context.venv
-                try:
-                    self.context.venv = venv
-                    func(self.context, *bound.args, **bound.kwargs)
-                finally:
-                    self.context.venv = previous_venv
+            venv = VirtualEnv(ctx=self.context, **self.venv_config)
         else:
-            func(self.context, *bound.args, **bound.kwargs)
+            venv = nullcontext()
+        with venv:
+            previous_venv = self.context.venv
+            try:
+                self.context.venv = venv
+                func(self.context, *bound.args, **bound.kwargs)
+            finally:
+                self.context.venv = previous_venv
 
 
 def command_group(
@@ -657,7 +715,7 @@ def command_group(
     help: str,
     description: str | None = None,
     venv_config: VirtualEnvConfig | None = None,
-    parent: CommandGroup | None = None,
+    parent: CommandGroup | str | list[str] | tuple[str] | None = None,
 ) -> CommandGroup:
     """
     Create a new command group.
